template<typename A>
class Rep0n: public parser<Rep0n<A> >
{
    typedef parser<Rep0n<A> > base_t;
    const A opt;
public:
    Rep0n(const parser<A>& a) : base_t(), opt(a.derived()) { }

    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        while(!scan.eof() && opt.parse(scan));
        return true;
    }

    /* nothing really expected */
    inline const std::string what() const { return "";/*opt.what();*/ }
};

template <typename A>
inline const Rep0n<A> operator*(const parser<A>& a) { return Rep0n<A>(a); }

///////////////

template<typename A>
class Rep1n: public parser<Rep1n<A> >
{
    typedef parser<Rep1n<A> > base_t;
    const A opt;
public:
    Rep1n(const parser<A>& a) : base_t(),opt(a.derived()) { }

    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        if(!opt.parse(scan)) return false;
        while(!scan.eof() && opt.parse(scan));
        return true;
    }

    inline const std::string what() const { return opt.what(); }
};

template <typename A>
inline const Rep1n<A> operator+(const parser<A>& a) { return Rep1n<A>(a); }

///////////////

template<typename A>
class Rep01: public parser<Rep01<A> >
{
    typedef parser<Rep01<A> > base_t;
    const A opt;
public:
    Rep01(const parser<A>& a) : base_t(),opt(a.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        opt.parse(scan);
        return true;
    }

    inline const std::string what() const { return opt.what(); }
};

template <typename A>
inline const Rep01<A> operator!(const parser<A>& a) { return Rep01<A>(a); }

///////////////

template<typename A, typename B>
class Alt: public parser<Alt<A,B> >
{
    typedef parser<Alt<A,B> > base_t;
    const A opt1;
    const B opt2;
public:
    Alt(const parser<A>& a, const parser<B>& b)
        : base_t(), opt1(a.derived()), opt2(b.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        return opt1.parse(scan) || opt2.parse(scan);
    }

    const std::string what() const { return base_t::what2join(opt1.what(), opt2.what(), " or "); }
};

GenBinaryOps(Alt, |)

///////////////

template<typename A, typename B>
class Seq: public parser<Seq<A,B> >
{
    typedef parser<Seq<A,B> > base_t;
    const A opt1;
    const B opt2;
public:
    Seq(const parser<A>& a, const parser<B>& b)
        : base_t(), opt1(a.derived()), opt2(b.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        typename ScannerT::iterator it = scan.pos();
        if(opt1.parse(scan) && opt2.parse(scan)) return true;
        scan.revert(it);
        return false;
    }

    const std::string what() const { return base_t::what2join(opt1.what(), opt2.what(), " followed by "); }
};

GenBinaryOps(Seq, >>)

///////////////

template<typename A, typename B>
class Union: public parser<Union<A,B> >
{
    typedef parser<Union<A,B> > base_t;
    const A opt1;
    const B opt2;
public:
    Union(const parser<A>& a, const parser<B>& b)
        : base_t(), opt1(a.derived()), opt2(b.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        typename ScannerT::iterator it = scan.pos();
        if(!opt1.parse(scan)) return false;
        scan.revert(it);
        return opt2.parse(scan);
    }

    const std::string what() const { return base_t::what2join(opt1.what(), opt2.what(), " and "); }
};
template<typename A, typename B>
class Difference: public parser<Difference<A,B> >
{
    typedef parser<Difference<A,B> > base_t;
    const A opt1;
    const B opt2;
public:
    Difference(const parser<A>& a, const parser<B>& b)
        : base_t(), opt1(a.derived()), opt2(b.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        typename ScannerT::iterator it = scan.pos();
        if(opt2.parse(scan)) { scan.revert(it); return false;}
        return opt1.parse(scan);
    }

    const std::string what() const
    {
        const std::string s = opt1.what();
        if(s.empty()) return "not " + opt2.what();
        return opt1.what() + " but not " + opt2.what();
    }
};
template<typename A, typename B>
class Xor: public parser<Xor<A,B> >
{
    typedef parser<Xor<A,B> > base_t;
    const A opt1;
    const B opt2;
public:
    Xor(const parser<A>& a, const parser<B>& b)
        : base_t(), opt1(a.derived()), opt2(b.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        typename ScannerT::iterator it = scan.pos();
        if(opt1.parse(scan))
        {
            if(opt2.parse(scan)) { scan.revert(it); return false; }
            return true;
        }
        return opt2.parse(scan);
    }

    const std::string what() const { return opt1.what() + " or " + opt2.what(); }
};
template<typename A, typename B>
class InSeq: public parser<InSeq<A,B> >
{
    typedef parser<InSeq<A,B> > base_t;
    const A opt1;
    const B opt2;
public:
    InSeq(const parser<A>& a, const parser<B>& b)
        : base_t(), opt1(a.derived()), opt2(b.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        bool a = opt1.parse(scan); // no backtracking
        bool b = opt2.parse(scan);
        return a || b; // no shortcircuiting
    }

    const std::string what() const { return opt1.what() + " or " + opt2.what(); }
};

GenBinaryOps(Union, &)
GenBinaryOps(Difference, &)
GenBinaryOps(Xor, ^)
GenBinaryOps(InSeq, ||)

///////////////

template<typename A, typename B>
class MSeq: public parser<MSeq<A,B> >
{
    typedef parser<MSeq<A,B> > base_t;
    const A opt1;
    const B opt2;
public:
    MSeq(const parser<A>& a, const parser<B>& b)
        : base_t(), opt1(a.derived()), opt2(b.derived()) { }
    template<typename ScannerT>
    bool parse(ScannerT& scan) const
    {
        if(!opt1.parse(scan)) return false;
        for(;;)
        {
            typename ScannerT::iterator it = scan.pos();
            if(!opt2.parse(scan)
            || !opt1.parse(scan)) { scan.revert(it); break; }
        }
        return true;
    }

    const std::string what() const { return opt1.what(); }
};

GenBinaryOps(MSeq, %)
