#include <cstdlib>
#include <unistd.h> // sleep

#include "exec.hh"
#include "error.hh"
#include "gosub.hh"
#include "stream.hh"
#include "gfx.hh"

namespace
{
    std::string cmdline;
}

void B_LibInit(int argc, char** argv)
{
    for(int a=1; a<argc; ++a)
    {
        if(a > 1) cmdline += ' ';
        cmdline += argv[a];
    }
    GFX_Init();
}

void B_LibDone()
{
    GFX_Done();
}

void B_StmtEND(int code)
{
    B_LibDone();
    std::exit(code);

    fprintf(stderr, "Eh, huh? std::exit returned?\n"); fflush(stderr);
}

void B_StmtSYSTEM(int code)
{
    B_StmtEND(code);
}

void B_StmtSTOP()
{
    B_LibDone();
    std::abort();
}

void B_CLEAR(int ds_size,int,int ss_size, bool in_sub)
{
    if(in_sub)
    {
        B_ERROR(5);
        // QuickBASIC throws "illegal function call"
        // if CLEAR called from inside a SUB. Not sure why.
    }

    ds_size=ds_size; // ignored
    ss_size=ss_size; // ignored

    GosubStack.clear(); // Defined to do so in QB manual

    B_StmtCLOSE(); // Close all open files

    ON_ERROR_GOTO_NOWHERE();

    /* This clears possible active error. */
    if(B_ERR_VALUE)
    {
        B_RESUME_TO(cont);
    cont:;
    }
}

void B_DummyLoop(unsigned long n_iterations)
{
    /* If the code uses dummy loops, let's
     * pretend we're a slow computer.
     * This makes Nibbles play nice.
     */
    usleep(n_iterations * 100);
}

const BAS_STR B_FuncVARPTR_S(int typetag, unsigned long offs)
{
    BAS_STR result;
    result += (char)typetag;
    for(unsigned n=0; n<sizeof(offs); ++n)
    {
        result += (char)( (offs >> (n*8)) & 0xFF );
    }
    return result;
}

long double VARPTR_ReadNum(const char* s)
{
    unsigned long offs = 0;
    for(unsigned n=0; n<sizeof(offs); ++n)
    {
        offs |= ((unsigned long)((unsigned char)s[n+1])) << (n*8);
    }
    switch( (unsigned char) s[0])
    {
        case VARPTR_TAG_SGN: return *reinterpret_cast<const BAS_SGN*>(offs);
        case VARPTR_TAG_DBL: return *reinterpret_cast<const BAS_DBL*>(offs);

        case VARPTR_TAG_BYTE: return *reinterpret_cast<const BAS_BYTE*>(offs);
        case VARPTR_TAG_INT: return *reinterpret_cast<const BAS_INT*>(offs);
        case VARPTR_TAG_LNG: return *reinterpret_cast<const BAS_LNG*>(offs);

        case VARPTR_TAG_UBYTE: return *reinterpret_cast<const BAS_UBYTE*>(offs);
        case VARPTR_TAG_UINT: return *reinterpret_cast<const BAS_UINT*>(offs);
        case VARPTR_TAG_ULONG: return *reinterpret_cast<const BAS_ULONG*>(offs);

        default: B_ERROR(5);
    }
    return 0;
}

const BAS_STR& VARPTR_ReadStr(const char* s)
{
    unsigned long offs = 0;
    for(unsigned n=0; n<sizeof(offs); ++n)
    {
        offs |= ((unsigned long)((unsigned char)s[n+1])) << (n*8);
    }
    switch(s[0])
    {
        case VARPTR_TAG_STR: break;
        default: B_ERROR(5);
    }
    return *reinterpret_cast<const BAS_STR*>(offs);
}

const std::string& B_FuncCOMMAND_S()
{
    return cmdline;
}

