#include <vector>
#include <utility>

#include "gfx_api.hh"

/* How palettes and colors work */
enum { CM_Mono=1,  // Mono:      0..1 colors
       CM_CGA=2,   // CGA:       0..3 colors
       CM_EGA=3,   // EGA:       0..15 colors
       CM_S10=4,   // Screen 10: 0..3 colors (off,on,blink,bright)
       CM_S13=5,   // Screen 13: 0..255 colors
       CM_RGB32=6
     };
enum { PM_Mono=10,    // Mono:      no palette
       PM_S10=20,     // Screen 10: 0..8 colors (/3 = blinkfrom, %3 = blinkto) (off,on,bright)
       PM_EGA=30,     // EGA:       16 colors
       PM_EGA64=40,   // EGA:       64 colors (ega8color + ega8color (sum))
       PM_VGA=50,     // VGA:       64*64*64 colors
     };

class GraphicScreen: public Screen
{
private:
    unsigned cpmode;
    unsigned ncolors;
    unsigned npalette;
    int modenumber;
private:
    struct Page
    {
        std::vector<unsigned short> contents;
        std::pair<int,int> cursorpos;
    };
    std::vector<Page> pages;
    int apage, vpage;
public:
    GraphicScreen();

    virtual ~GraphicScreen();

    virtual std::pair<int,int> GetCursorPos();
    virtual void SetCursorPos(const std::pair<int,int>& newpos);
    virtual void SetCursorVisibility(bool visible);
    virtual void SetCursorSize(int start, int stop);
    virtual void SetPages(int npages);
    virtual void SetAPage(int pageno);
    virtual void SetVPage(int pageno);
    virtual std::pair<int,int> GetWinSize();
    virtual std::pair<int,int> ReadScreen(int x,int y);

    virtual void SetWinSize(const std::pair<int,int>& newsize, bool approximate);
    virtual void OutS(const char* buffer, size_t nbytes);
    virtual void SetBorderColor(int border);

    virtual void ScreenPOKE(int x,int y, int colorflag, int value);
    virtual void ClearEntireScreen();
    virtual void ClearText(int firstline, int lastline);

    virtual void SetGFXresolution(int xs,int ys, unsigned cpmode);

    virtual void ValidateColor(int color)
    {
        if(color < 0 || color >= (int)ncolors) B_ERROR(5);
    }

    virtual void FailIfNotGraphicsMode() { }
    virtual void SetPalette(int index, int value);
    virtual void SetPixel(int x,int y, unsigned color);
    virtual void Hline(int x1,int x2, int y, unsigned color);
    virtual void Vline(int y1,int y2, int x, unsigned color);
    virtual unsigned GetPixel(int x,int y);

    virtual int GetModeNumber() { return modenumber; }
    virtual void SetModeNumber(int m) { modenumber = m; }
};
