#include <qblib/qblib.hh>

#include <stdarg.h>
#include <vector>
#include <utility>

#include <qblib/error.hh>

template<typename T>
class BAS_MAGICARRAY_ImplBase: public ptrable
{
public:
    virtual ~BAS_MAGICARRAY_ImplBase() {}

    virtual T& Index(unsigned numdims, va_list ap) = 0;
    virtual const T& Index(unsigned numdims, va_list ap) const = 0;

    virtual void Redim(unsigned numdims, va_list ap) = 0;
    virtual void Erase() = 0;
    virtual long Lbound(unsigned dimindex=1) const = 0;
    virtual long Ubound(unsigned dimindex=1) const = 0;
};

template<typename T,unsigned Ndims>
class BAS_DYNARRAY;

template<typename T>
class BAS_MAGICARRAY
{
public:
    // Initializing with a static array:
    BAS_MAGICARRAY(T* data, unsigned numdims, ...);

    // Initializing with a dynamic array:
    template<unsigned numdims>
    BAS_MAGICARRAY(BAS_DYNARRAY<T,numdims>& data);

    // Initializing with reference to a kindsman:
    BAS_MAGICARRAY(BAS_MAGICARRAY& data) : impl(data.impl) { }

    T& Index(unsigned numdims, ...);
    const T& Index(unsigned numdims, ...) const;

    void Redim(unsigned numdims, ...);
    void Erase()                         { impl->Erase(); }
    long Lbound(unsigned dimindex=1) const { return impl->Lbound(dimindex); }
    long Ubound(unsigned dimindex=1) const { return impl->Ubound(dimindex); }
private:
    autoptr<BAS_MAGICARRAY_ImplBase<T> > impl;
};

template<typename T>
class BAS_MAGICARRAY_ImplSTATIC: public BAS_MAGICARRAY_ImplBase<T>
{
public:
    BAS_MAGICARRAY_ImplSTATIC(T* d, unsigned n)
        : data(d), num_dims(n), dims(n) { }
    void InitDims(va_list ap);
public:
    virtual T& Index(unsigned numdims, va_list ap);
    virtual const T& Index(unsigned numdims, va_list ap) const;

    virtual void Redim(unsigned, va_list)
        { B_ERROR(10); /* duplicate definition */ }
    virtual void Erase();
    virtual long Lbound(unsigned dimindex=1) const;
    virtual long Ubound(unsigned dimindex=1) const;
protected:
    unsigned Volume() const;
protected:
    T* data;
    unsigned num_dims;
    std::vector<std::pair<int/*begin*/, unsigned/*dim*/> > dims;
};

template<typename T, unsigned Ndims>
class BAS_MAGICARRAY_ImplDYNAMIC: public BAS_MAGICARRAY_ImplBase<T>
{
public:
    BAS_MAGICARRAY_ImplDYNAMIC(BAS_DYNARRAY<T,Ndims>& d)
        : data(d) { }

    virtual T& Index(unsigned numdims, va_list ap)
    {
        return data.DoIndex(numdims, ap);
    }
    virtual const T& Index(unsigned numdims, va_list ap) const
    {
        return data.DoIndex(numdims, ap);
    }
    virtual void Redim(unsigned n, va_list ap) { data.DoRedim(n,ap); }
    virtual void Erase() { data.Erase() ;}
    virtual long Lbound(unsigned dimindex=1) const { return data.Lbound(dimindex); }
    virtual long Ubound(unsigned dimindex=1) const { return data.Ubound(dimindex); }
private:
    BAS_DYNARRAY<T,Ndims>& data;
};

#include <qblib/magicarray.tcc>
