#include <cctype>

#include "stringfun.hh"
#include "error.hh"

const std::string RTrim(const std::string& b, char with)
{
    std::size_t size = b.size();
    while(size > 0 && b[size-1] == with)--size;
    return b.substr(0, size);
}

const std::string LTrim(const std::string& b, char with)
{
    std::size_t strip=0;
    for(std::size_t a=0; a<b.size() && b[a]==with; ++a)++strip;
    return b.substr(strip);
}

const std::string RTrim(const std::string& s)
{
    std::size_t q = 0; /* Number of chars to eat */
    while(q < s.size() && (s[s.size()-q-1] == ' '
                        || s[s.size()-q-1] == '\t')) ++q;
    return s.substr(0, s.size()-q);
}

const std::string LTrim(const std::string& s)
{
    std::size_t p = 0;
    while(p < s.size() && (s[p] == ' ' || s[p] == '\t')) ++p;
    return s.substr(p);
}

const std::string Trim(const std::string& s)
{
    std::size_t p = 0;
    while(p < s.size() && (s[p] == ' ' || s[p] == '\t')) ++p;

    std::size_t q = 0;
    while((p+q) < s.size() && (s[s.size()-q-1] == ' ' || s[s.size()-q-1] == '\t')) ++q;

    return std::string(s.data()+p, s.size()-p-q);
}

const std::string LCase(const std::string& s)
{
    std::string result = s;
    for(std::size_t a=0; a<result.size(); ++a)
        result[a] = std::tolower(result[a]);
    return result;
}

const std::string UCase(const std::string& s)
{
    std::string result = s;
    for(std::size_t a=0; a<result.size(); ++a)
        result[a] = std::toupper(result[a]);
    return result;
}

void B_StmtLSET(std::string& result, const std::string& input)
{
    std::size_t copy_length = std::min( result.size(), input.size() );
    std::size_t pad_length  = std::max( result.size(), input.size() );

    result.replace(0, copy_length, input, 0, copy_length);
    result.replace(copy_length, pad_length-copy_length, pad_length-copy_length, ' ');
}

void B_StmtRSET(std::string& result, const std::string& input)
{
    std::size_t copy_length = std::min( result.size(), input.size() );
    std::size_t pad_length  = std::max( result.size(), input.size() );

    result.replace(pad_length-copy_length, copy_length, input, 0, copy_length);
    result.replace(0, pad_length-copy_length, pad_length-copy_length, ' ');

}

void B_StmtMID_S(std::string& target, int offset, int number, const std::string& input)
{
    if(offset <= 0 || number < 0 || offset+number > target.size()) B_ERROR(5); // illegal function call
    target.replace(offset-1, number, input);
}
