#include "symtables.hh"
#include "binding.hh"
#include "basfun.hh"
#include "stmt.hh"

const TypeP TypeInt = new TypeInfo(TypeType::Integer);
const TypeP TypeSng = new TypeInfo(TypeType::Single);
const TypeP TypeDbl = new TypeInfo(TypeType::Double);
const TypeP TypeLng = new TypeInfo(TypeType::Long);
const TypeP TypeStr = new TypeInfo(TypeType::String);

const TypeP TypeByte  = new TypeInfo(TypeType::Byte);
const TypeP TypeUbyte = new TypeInfo(TypeType::UnsignedByte);
const TypeP TypeUint  = new TypeInfo(TypeType::UnsignedInteger);
const TypeP TypeUlong = new TypeInfo(TypeType::UnsignedLong);

bool TypeInfo::operator==(const TypeInfo& b) const
{
    return Type==b.Type
        && arraytype==b.arraytype
        && fixlength==b.fixlength
        && dimensions==b.dimensions
        && !members==!b.members
        && (!members || *members == *b.members)
      ;
}

bool UserType::operator==(const UserType& b)const
{
    for(std::size_t a=0; a<size(); ++a)
        if( (*this)[a].first   != b[a].first
        || !(*this)[a].second  != !b[a].second
        || (b[a].second && *(*this)[a].second != *b[a].second)
          )
            return false;

     return Names == b.Names;
}

bool UserType::DeclareMember(const std::string& name, TypeP p)
{
    const std::string basename = BasUCASE(StripTypeSuffix(name));

    if(Names.find(basename) != Names.end()) return false;
    Names.insert(std::make_pair(basename, size()));
    (*this).push_back(std::make_pair(basename, p));
    return true;
}

TypeP UserType::GetMember(const std::string& name) const
{
    const std::string basename = BasUCASE(StripTypeSuffix(name));

    auto i = Names.find(basename);
    if(i != Names.end())
    {
        auto a = i->second;
        if(a < size())
            if( (*this)[a].first == basename) return (*this)[a].second;
    }
    return 0;
}

const std::string TypeInfo::Dump() const
{
    std::string res;
    switch(Type)
    {
        case TypeType::Integer: res += "INTEGER"; break;
        case TypeType::Long:    res += "LONG"; break;
        case TypeType::Single:  res += "SINGLE"; break;
        case TypeType::Double:  res += "DOUBLE"; break;
        case TypeType::String:  res += "STRING"; break;

        case TypeType::Byte:             res += "_BYTE"; break;
        case TypeType::UnsignedByte:     res += "_UNSIGNED _BYTE"; break;
        case TypeType::UnsignedInteger:  res += "_UNSIGNED INTEGER"; break;
        case TypeType::UnsignedLong:     res += "_UNSIGNED LONG"; break;

        case TypeType::UserType:
        {
            res += "{";
            if(!members) res += "NULL";
            else for(std::size_t a=0; a<members->size(); ++a)
            {
                if(a) res += ',';
                res += (*members)[a].first;
                res += ':';
                TypeP t = (*members)[a].second;
                res += t ? t->Dump() : "NULL";
            }
            res += "}";
            break;
        }
    }
    if(fixlength) { char Buf[64];sprintf(Buf, "*%u", fixlength); res += Buf; }

    switch(arraytype)
    {
        case TypeInfo::DynamicArray: res = "$DYNAMIC " + res; goto DoArray;
        case TypeInfo::StaticArray: res = "$STATIC " + res; goto DoArray;
        case TypeInfo::ParamArray: res = "$MAGICAL " + res; goto DoArray;
        DoArray:
            res += "(";
            for(unsigned a=0; a<dimensions.size(); ++a)
            {
                if(a>0) res += ",";
                char Buf[64];
                sprintf(Buf, "%ld TO %ld",
                    (long)(dimensions[a].first),
                    (long)(dimensions[a].first + dimensions[a].second - 1));
                res += Buf;
            }
            res += ")";
            break;
       case TypeInfo::NotArray: ;
    }
    return res;
}

